import argparse

from . import doc_split, usage, version
from .playlist import PlayDB


def add(files):
    """Add files in the current directory to the database."""
    PlayDB().add(files)


def remove(files):
    """Remove files in the current directory from the database."""
    PlayDB().remove(files)


def set(filename):
    """Set the current file."""
    PlayDB().set(filename)


def current():
    """Get the current file."""
    print PlayDB().current()


def next():
    """Go to the next file."""
    db = PlayDB()
    if db.next():
        print db.current()


def show():
    """Show the database for the current directory."""
    PlayDB().show()


def config(config_string):
    """Set configuration for the current directory."""
    PlayDB().config(config_string)


def show_config():
    """Show configuration for the current directory."""
    PlayDB().show_config()


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=usage[0], epilog=usage[1])
    parser.add_argument('-v', action='version', version=version(parser.prog))
    subparsers = parser.add_subparsers(dest='subcommand')

    subparser = subparsers.add_parser('add', description=doc_split(add))
    subparser.add_argument(
        'files', metavar='FILE', type=str, nargs='+', help='Files to add.')
    subparser.set_defaults(func=add)

    subparser = subparsers.add_parser('remove', description=doc_split(remove))
    subparser.add_argument(
        'files', metavar='FILE', type=str, nargs='+', help='Files to remove')
    subparser.set_defaults(func=remove)

    subparser = subparsers.add_parser('set', description=doc_split(set))
    subparser.add_argument(
        'filename', metavar='FILE', type=str, help='New current file.')
    subparser.set_defaults(func=set)

    subparser = subparsers.add_parser(
        'current', description=doc_split(current))
    subparser.set_defaults(func=current)

    subparser = subparsers.add_parser('next', description=doc_split(next))
    subparser.set_defaults(func=next)

    subparser = subparsers.add_parser('show', description=doc_split(show))
    subparser.set_defaults(func=show)

    subparser = subparsers.add_parser('config', description=doc_split(config))
    subparser.add_argument(
        'config_string', metavar='CONFIG', type=str,
        help='Configuration string.')
    subparser.set_defaults(func=config)

    subparser = subparsers.add_parser(
        'show_config', description=doc_split(show_config))
    subparser.set_defaults(func=show_config)

    args = parser.parse_args()

    args.func(**{k: v for k, v in vars(args).items()
        if k not in ('func', 'subcommand')})


if __name__ == '__main__':
    main()



"""playlist: Keep track of on disk playlists.


Copyright (c) 2017 Jeroen F.J. Laros <jlaros@fixedpoint.nl>

Licensed under the MIT license, see the LICENSE file.
"""
from .playlist import PlayDB


__version_info__ = ('0', '0', '3')

__version__ = '.'.join(__version_info__)
__author__ = 'Jeroen F.J. Laros'
__contact__ = 'jlaros@fixedpoint.nl'
__homepage__ = 'https://github.com/jfjlaros/playlist'

usage = __doc__.split('\n\n\n')


def doc_split(func):
    return func.__doc__.split('\n\n')[0]


def version(name):
    return '{} version {}\n\nAuthor   : {} <{}>\nHomepage : {}'.format(
        name, __version__, __author__, __contact__, __homepage__)



import os

import yaml


class PlayDB(object):
    def __init__(self):
        config_path = '{}/.cache/playlist'.format(os.getenv('HOME'))

        self._config = '{}/db.yml'.format(config_path)
        if not os.path.isfile(self._config):
            os.makedirs(config_path)
            yaml.safe_dump({}, open(self._config, 'w'))

        self._db = yaml.safe_load(open(self._config, 'r'))

        self._path = os.path.realpath('.')
        if self._path not in self._db:
            self._db[self._path] = {'files': [], 'offset': 0, 'config': ''}

        self._item = self._db[self._path]

    def __del__(self):
        yaml.safe_dump(
            self._db, open(self._config, 'w'), width=76,
            default_flow_style=False)

    def _overflow(self):
        return self._item['offset'] >= len(self._item['files'])

    def add(self, files):
        for file_name in files:
            if not file_name.startswith('.') and os.path.isfile(file_name):
                if file_name not in self._item['files']:
                    self._item['files'].append(file_name)

    def remove(self, files):
        for index, file_name in enumerate(self._item['files']):
            if file_name in files:
                self._item['files'].pop(index)
                if index < self._item['offset']:
                    self._item['offset'] -= 1

    def set(self, filename):
        for index, file_name in enumerate(self._item['files']):
            if file_name == filename:
                self._item['offset'] = index
                return

    def current(self):
        if not self._item['offset']:
            return ''
        return self._item['files'][self._item['offset'] - 1]

    def next(self):
        if self._overflow():
            return False
        self._item['offset'] += 1
        return True

    def show(self):
        print 'Playlist:'
        for index, file_name in enumerate(self._item['files']):
            spacer = ' '
            if index == self._item['offset']:
                spacer = '*'
            print '{} {}'.format(spacer, file_name)
        print '\nConfig: {}'.format(self._item['config'])

    def config(self, config_string):
        self._item['config'] = config_string

    def show_config(self):
        print self._item['config']



import os
import sys

from setuptools import setup


package = 'playlist'
package_name = 'playlistdb'
description = '{}: Keep track of on disk playlists.'.format(package_name)
documentation = 'README.rst'
license = 'MIT License'
keywords = []

dependencies = ['pyyaml']
develop_dependencies = ['fake-open', 'pytest', 'tox']
supported = [(2, 7), (3, 3), (3, 4)]
classifiers = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
    'Topic :: Scientific/Engineering',
]


if sys.version_info < supported[0]:
    raise Exception('{} requires Python {}.{} or higher.'.format(
        package, *supported[0]))

if sys.version_info[:2] == supported[0]:
    dependencies.extend(['argparse', 'importlib'])

# This is quite the hack, but we don't want to import our package from here
# since that's recipe for disaster (it might have some uninstalled
# dependencies, or we might import another already installed version).
distmeta = {}
for line in open(os.path.join(package, '__init__.py')):
    try:
        field, value = (x.strip() for x in line.split('='))
    except ValueError:
        continue
    if field == '__version_info__':
        value = value.strip('[]()')
        value = '.'.join(x.strip(' \'"') for x in value.split(','))
    else:
        value = value.strip('\'"')
    distmeta[field] = value

try:
    with open(documentation) as readme:
        long_description = readme.read()
except IOError:
    long_description = 'See ' + distmeta['__homepage__']

language_string = 'Programming Language :: Python'
classifiers += [
    'License :: OSI Approved :: {}'.format(license),
    'Operating System :: OS Independent',
    language_string,
    '{} :: {}'.format(language_string, supported[0][0]),
    '{} :: {}'.format(language_string, supported[-1][0])] + \
    ['{} :: {}.{}'.format(language_string, *version) for version in supported]

setup(
    name=package_name,
    version=distmeta['__version_info__'],
    description=description,
    long_description=long_description,
    author=distmeta['__author__'],
    author_email=distmeta['__contact__'],
    url=distmeta['__homepage__'],
    license=license,
    platforms=['any'],
    packages=[package],
    install_requires=dependencies,
    tests_require=develop_dependencies,
    entry_points={
        'console_scripts': ['{0} = {0}.cli:main'.format(package)]
    },
    classifiers=classifiers,
    keywords=' '.join(keywords)
)
